package org.apache.bcel.generic;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.apache.bcel.classfile.AnnotationEntry;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.ElementValuePair;
import org.apache.bcel.classfile.RuntimeInvisibleAnnotations;
import org.apache.bcel.classfile.RuntimeInvisibleParameterAnnotations;
import org.apache.bcel.classfile.RuntimeVisibleAnnotations;
import org.apache.bcel.classfile.RuntimeVisibleParameterAnnotations;

public class AnnotationEntryGen {
   static final AnnotationEntryGen[] EMPTY_ARRAY = new AnnotationEntryGen[0];
   private int typeIndex;
   private List<ElementValuePairGen> evs;
   private final ConstantPoolGen cpool;
   private boolean isRuntimeVisible;

   static Attribute[] getAnnotationAttributes(ConstantPoolGen cp, AnnotationEntryGen[] annotationEntryGens) {
      if (annotationEntryGens.length == 0) {
         return Attribute.EMPTY_ARRAY;
      } else {
         try {
            int countVisible = 0;
            int countInvisible = 0;
            AnnotationEntryGen[] var4 = annotationEntryGens;
            int var5 = annotationEntryGens.length;

            for(int var6 = 0; var6 < var5; ++var6) {
               AnnotationEntryGen a = var4[var6];
               if (a.isRuntimeVisible()) {
                  ++countVisible;
               } else {
                  ++countInvisible;
               }
            }

            ByteArrayOutputStream rvaBytes = new ByteArrayOutputStream();
            ByteArrayOutputStream riaBytes = new ByteArrayOutputStream();
            DataOutputStream rvaDos = new DataOutputStream(rvaBytes);
            Throwable var47 = null;

            try {
               DataOutputStream riaDos = new DataOutputStream(riaBytes);
               Throwable var9 = null;

               try {
                  rvaDos.writeShort(countVisible);
                  riaDos.writeShort(countInvisible);
                  AnnotationEntryGen[] var10 = annotationEntryGens;
                  int var11 = annotationEntryGens.length;

                  for(int var12 = 0; var12 < var11; ++var12) {
                     AnnotationEntryGen a = var10[var12];
                     if (a.isRuntimeVisible()) {
                        a.dump(rvaDos);
                     } else {
                        a.dump(riaDos);
                     }
                  }
               } catch (Throwable var37) {
                  var9 = var37;
                  throw var37;
               } finally {
                  if (riaDos != null) {
                     if (var9 != null) {
                        try {
                           riaDos.close();
                        } catch (Throwable var36) {
                           var9.addSuppressed(var36);
                        }
                     } else {
                        riaDos.close();
                     }
                  }

               }
            } catch (Throwable var39) {
               var47 = var39;
               throw var39;
            } finally {
               if (rvaDos != null) {
                  if (var47 != null) {
                     try {
                        rvaDos.close();
                     } catch (Throwable var35) {
                        var47.addSuppressed(var35);
                     }
                  } else {
                     rvaDos.close();
                  }
               }

            }

            byte[] rvaData = rvaBytes.toByteArray();
            byte[] riaData = riaBytes.toByteArray();
            int rvaIndex = -1;
            int riaIndex = -1;
            if (rvaData.length > 2) {
               rvaIndex = cp.addUtf8("RuntimeVisibleAnnotations");
            }

            if (riaData.length > 2) {
               riaIndex = cp.addUtf8("RuntimeInvisibleAnnotations");
            }

            List<Attribute> newAttributes = new ArrayList();
            if (rvaData.length > 2) {
               newAttributes.add(new RuntimeVisibleAnnotations(rvaIndex, rvaData.length, new DataInputStream(new ByteArrayInputStream(rvaData)), cp.getConstantPool()));
            }

            if (riaData.length > 2) {
               newAttributes.add(new RuntimeInvisibleAnnotations(riaIndex, riaData.length, new DataInputStream(new ByteArrayInputStream(riaData)), cp.getConstantPool()));
            }

            return (Attribute[])newAttributes.toArray(Attribute.EMPTY_ARRAY);
         } catch (IOException var41) {
            IOException e = var41;
            System.err.println("IOException whilst processing annotations");
            e.printStackTrace();
            return null;
         }
      }
   }

   static Attribute[] getParameterAnnotationAttributes(ConstantPoolGen cp, List<AnnotationEntryGen>[] vec) {
      int[] visCount = new int[vec.length];
      int totalVisCount = 0;
      int[] invisCount = new int[vec.length];
      int totalInvisCount = 0;

      try {
         for(int i = 0; i < vec.length; ++i) {
            if (vec[i] != null) {
               Iterator var7 = vec[i].iterator();

               while(var7.hasNext()) {
                  AnnotationEntryGen element = (AnnotationEntryGen)var7.next();
                  int var10002;
                  if (element.isRuntimeVisible()) {
                     var10002 = visCount[i]++;
                     ++totalVisCount;
                  } else {
                     var10002 = invisCount[i]++;
                     ++totalInvisCount;
                  }
               }
            }
         }

         ByteArrayOutputStream rvaBytes = new ByteArrayOutputStream();
         DataOutputStream rvaDos = new DataOutputStream(rvaBytes);
         Throwable var44 = null;

         try {
            rvaDos.writeByte(vec.length);

            for(int i = 0; i < vec.length; ++i) {
               rvaDos.writeShort(visCount[i]);
               if (visCount[i] > 0) {
                  Iterator var10 = vec[i].iterator();

                  while(var10.hasNext()) {
                     AnnotationEntryGen element = (AnnotationEntryGen)var10.next();
                     if (element.isRuntimeVisible()) {
                        element.dump(rvaDos);
                     }
                  }
               }
            }
         } catch (Throwable var37) {
            var44 = var37;
            throw var37;
         } finally {
            if (rvaDos != null) {
               if (var44 != null) {
                  try {
                     rvaDos.close();
                  } catch (Throwable var34) {
                     var44.addSuppressed(var34);
                  }
               } else {
                  rvaDos.close();
               }
            }

         }

         ByteArrayOutputStream riaBytes = new ByteArrayOutputStream();
         DataOutputStream riaDos = new DataOutputStream(riaBytes);
         Throwable var46 = null;

         int rvaIndex;
         try {
            riaDos.writeByte(vec.length);

            for(rvaIndex = 0; rvaIndex < vec.length; ++rvaIndex) {
               riaDos.writeShort(invisCount[rvaIndex]);
               if (invisCount[rvaIndex] > 0) {
                  Iterator var50 = vec[rvaIndex].iterator();

                  while(var50.hasNext()) {
                     AnnotationEntryGen element = (AnnotationEntryGen)var50.next();
                     if (!element.isRuntimeVisible()) {
                        element.dump(riaDos);
                     }
                  }
               }
            }
         } catch (Throwable var35) {
            var46 = var35;
            throw var35;
         } finally {
            if (riaDos != null) {
               if (var46 != null) {
                  try {
                     riaDos.close();
                  } catch (Throwable var33) {
                     var46.addSuppressed(var33);
                  }
               } else {
                  riaDos.close();
               }
            }

         }

         byte[] rvaData = rvaBytes.toByteArray();
         byte[] riaData = riaBytes.toByteArray();
         rvaIndex = -1;
         int riaIndex = -1;
         if (totalVisCount > 0) {
            rvaIndex = cp.addUtf8("RuntimeVisibleParameterAnnotations");
         }

         if (totalInvisCount > 0) {
            riaIndex = cp.addUtf8("RuntimeInvisibleParameterAnnotations");
         }

         List<Attribute> newAttributes = new ArrayList();
         if (totalVisCount > 0) {
            newAttributes.add(new RuntimeVisibleParameterAnnotations(rvaIndex, rvaData.length, new DataInputStream(new ByteArrayInputStream(rvaData)), cp.getConstantPool()));
         }

         if (totalInvisCount > 0) {
            newAttributes.add(new RuntimeInvisibleParameterAnnotations(riaIndex, riaData.length, new DataInputStream(new ByteArrayInputStream(riaData)), cp.getConstantPool()));
         }

         return (Attribute[])newAttributes.toArray(Attribute.EMPTY_ARRAY);
      } catch (IOException var39) {
         IOException e = var39;
         System.err.println("IOException whilst processing parameter annotations");
         e.printStackTrace();
         return null;
      }
   }

   public static AnnotationEntryGen read(DataInput dis, ConstantPoolGen cpool, boolean b) throws IOException {
      AnnotationEntryGen a = new AnnotationEntryGen(cpool);
      a.typeIndex = dis.readUnsignedShort();
      int elemValuePairCount = dis.readUnsignedShort();

      for(int i = 0; i < elemValuePairCount; ++i) {
         int nidx = dis.readUnsignedShort();
         a.addElementNameValuePair(new ElementValuePairGen(nidx, ElementValueGen.readElementValue(dis, cpool), cpool));
      }

      a.isRuntimeVisible(b);
      return a;
   }

   public AnnotationEntryGen(AnnotationEntry a, ConstantPoolGen cpool, boolean copyPoolEntries) {
      this.cpool = cpool;
      if (copyPoolEntries) {
         this.typeIndex = cpool.addUtf8(a.getAnnotationType());
      } else {
         this.typeIndex = a.getAnnotationTypeIndex();
      }

      this.isRuntimeVisible = a.isRuntimeVisible();
      this.evs = this.copyValues(a.getElementValuePairs(), cpool, copyPoolEntries);
   }

   private AnnotationEntryGen(ConstantPoolGen cpool) {
      this.cpool = cpool;
   }

   public AnnotationEntryGen(ObjectType type, List<ElementValuePairGen> elements, boolean vis, ConstantPoolGen cpool) {
      this.cpool = cpool;
      this.typeIndex = cpool.addUtf8(type.getSignature());
      this.evs = elements;
      this.isRuntimeVisible = vis;
   }

   public void addElementNameValuePair(ElementValuePairGen evp) {
      if (this.evs == null) {
         this.evs = new ArrayList();
      }

      this.evs.add(evp);
   }

   private List<ElementValuePairGen> copyValues(ElementValuePair[] in, ConstantPoolGen cpool, boolean copyPoolEntries) {
      List<ElementValuePairGen> out = new ArrayList();
      ElementValuePair[] var5 = in;
      int var6 = in.length;

      for(int var7 = 0; var7 < var6; ++var7) {
         ElementValuePair nvp = var5[var7];
         out.add(new ElementValuePairGen(nvp, cpool, copyPoolEntries));
      }

      return out;
   }

   public void dump(DataOutputStream dos) throws IOException {
      dos.writeShort(this.typeIndex);
      dos.writeShort(this.evs.size());
      Iterator var2 = this.evs.iterator();

      while(var2.hasNext()) {
         ElementValuePairGen envp = (ElementValuePairGen)var2.next();
         envp.dump(dos);
      }

   }

   public AnnotationEntry getAnnotation() {
      AnnotationEntry a = new AnnotationEntry(this.typeIndex, this.cpool.getConstantPool(), this.isRuntimeVisible);
      Iterator var2 = this.evs.iterator();

      while(var2.hasNext()) {
         ElementValuePairGen element = (ElementValuePairGen)var2.next();
         a.addElementNameValuePair(element.getElementNameValuePair());
      }

      return a;
   }

   public int getTypeIndex() {
      return this.typeIndex;
   }

   public final String getTypeName() {
      return this.getTypeSignature();
   }

   public final String getTypeSignature() {
      ConstantUtf8 utf8 = (ConstantUtf8)this.cpool.getConstant(this.typeIndex);
      return utf8.getBytes();
   }

   public List<ElementValuePairGen> getValues() {
      return this.evs;
   }

   public boolean isRuntimeVisible() {
      return this.isRuntimeVisible;
   }

   private void isRuntimeVisible(boolean b) {
      this.isRuntimeVisible = b;
   }

   public String toShortString() {
      StringBuilder s = new StringBuilder();
      s.append("@").append(this.getTypeName()).append("(");

      for(int i = 0; i < this.evs.size(); ++i) {
         s.append(this.evs.get(i));
         if (i + 1 < this.evs.size()) {
            s.append(",");
         }
      }

      s.append(")");
      return s.toString();
   }

   public String toString() {
      StringBuilder s = new StringBuilder(32);
      s.append("AnnotationGen:[").append(this.getTypeName()).append(" #").append(this.evs.size()).append(" {");

      for(int i = 0; i < this.evs.size(); ++i) {
         s.append(this.evs.get(i));
         if (i + 1 < this.evs.size()) {
            s.append(",");
         }
      }

      s.append("}]");
      return s.toString();
   }
}
